#define METSmearing_cxx
// The class definition in METSmearing.h has been generated automatically
// by the ROOT utility TTree::MakeSelector(). This class is derived
// from the ROOT class TSelector. For more information on the TSelector
// framework see $ROOTSYS/README/README.SELECTOR or the ROOT User Manual.

// The following methods are defined in this file:
//    Begin():        called every time a loop on the tree starts,
//                    a convenient place to create your histograms.
//    SlaveBegin():   called after Begin(), when on PROOF called only on the
//                    slave servers.
//    Process():      called for each event, in this function you decide what
//                    to read and fill your histograms.
//    SlaveTerminate: called at the end of the loop on the tree, when on PROOF
//                    called only on the slave servers.
//    Terminate():    called at the end of the loop on the tree,
//                    a convenient place to draw/fit your histograms.
//
// To use this file, try the following session on your Tree T:
//
// Root > T->Process("METSmearing.C")
// Root > T->Process("METSmearing.C","some options")
// Root > T->Process("METSmearing.C+")
//

#include "METSmearing.h"
#include <iostream>
#include <TH1F.h>
#include <TLatex.h>

using namespace std;

void METSmearing::BuildEvent() {
  
  Muons.clear();
  for (int i = 0; i < NMuon; ++i) {
    MyMuon muon(Muon_Px[i], Muon_Py[i], Muon_Pz[i], Muon_E[i]);
      muon.SetIsolation(Muon_Iso[i]);
      muon.SetCharge(Muon_Charge[i]);
      Muons.push_back(muon);
   }
   
   Electrons.clear();
   for (int i = 0; i < NElectron; ++i) {
      MyElectron electron(Electron_Px[i], Electron_Py[i], Electron_Pz[i], Electron_E[i]);
      electron.SetIsolation(Electron_Iso[i]);
      electron.SetCharge(Electron_Charge[i]);
      Electrons.push_back(electron);
   }
   
   Photons.clear();
   for (int i = 0; i < NPhoton; ++i) {
      MyPhoton photon(Photon_Px[i], Photon_Py[i], Photon_Pz[i], Photon_E[i]);
      photon.SetIsolation(Photon_Iso[i]);
      Photons.push_back(photon);
   }
   
   Jets.clear();
   for (int i = 0; i < NJet; ++i) {
      MyJet jet(Jet_Px[i], Jet_Py[i], Jet_Pz[i], Jet_E[i]);
      jet.SetBTagDiscriminator(Jet_btag[i]);
      jet.SetJetID(Jet_ID[i]);
      Jets.push_back(jet);
   }
   
   hadB.SetXYZM(MChadronicBottom_px, MChadronicBottom_py, MChadronicBottom_pz, 4.8);
   lepB.SetXYZM(MCleptonicBottom_px, MCleptonicBottom_py, MCleptonicBottom_pz, 4.8);
   hadWq.SetXYZM(MChadronicWDecayQuark_px, MChadronicWDecayQuark_py, MChadronicWDecayQuark_pz, 0.0);
   hadWqb.SetXYZM(MChadronicWDecayQuarkBar_px, MChadronicWDecayQuarkBar_py, MChadronicWDecayQuarkBar_pz, 0.0);
   lepWl.SetXYZM(MClepton_px, MClepton_py, MClepton_pz, 0.0);
   lepWn.SetXYZM(MCneutrino_px, MCneutrino_py, MCneutrino_pz, 0.0);
   met.SetXYZM(MET_px, MET_py, 0., 0.);
   
   EventWeight *= weight_factor;
   
}

void METSmearing::Begin(TTree * /*tree*/) {
   // The Begin() function is called at the start of the query.
   // When running with PROOF Begin() is only called on the client.
   // The tree argument is deprecated (on PROOF 0 is passed).
   
   TString option = GetOption();
   
}

void METSmearing::SlaveBegin(TTree * /*tree*/) {
   // The SlaveBegin() function is called after the Begin() function.
   // When running with PROOF SlaveBegin() is called on each slave server.
   // The tree argument is deprecated (on PROOF 0 is passed).
   
   TString option = GetOption();
  

   h_MET = new TH1F("MET", "MET ", nbins, metMin,metMax);
   h_MET->SetXTitle("M_{ET} Z");
   h_MET->Sumw2();
   histograms.push_back(h_MET);

   h_METPerp = new TH1F("METPerp", "MET Perp",nbins, metParaMin,metParaMax);
   h_METPerp->SetXTitle("M_{ET} #perp");
   h_METPerp->Sumw2();
   histograms.push_back(h_METPerp);

   h_METPara = new TH1F("METPara", "MET Para", nbins, metParaMin,metParaMax);
   h_METPara->SetXTitle("M_{ET} #parallel");
   h_METPara->Sumw2();
   histograms.push_back(h_METPara);

   h_METx = new TH1F("METx", "MET x", nbins, metParaMin,metParaMax);
   h_METx->SetXTitle("M_{ET} x");
   h_METx->Sumw2();
   histograms.push_back(h_METx);

   h_METy = new TH1F("METy", "MET y", nbins, metParaMin,metParaMax);
   h_METy->SetXTitle("M_{ET} y");
   h_METy->Sumw2();
   histograms.push_back(h_METy);

   h_METSmeared = new TH1F("METSmeared", "MET Smeared", nbins, metMin,metMax);
   h_METSmeared->SetXTitle("M_{ET} Z");
   h_METSmeared->Sumw2();
   histogramsSmeared.push_back(h_METSmeared);

   h_METPerpSmeared = new TH1F("METPerpSmeared", "MET Perp smeared ",nbins, metParaMin,metParaMax);
   h_METPerpSmeared->SetXTitle("M_{ET} #perp");
   h_METPerpSmeared->Sumw2();
   histogramsSmeared.push_back(h_METPerpSmeared);

   h_METParaSmeared = new TH1F("METParaSmeared", "MET Para Smeared", nbins, metParaMin,metParaMax);
   h_METParaSmeared->SetXTitle("M_{ET} #parallel");
   h_METParaSmeared->Sumw2();
   histogramsSmeared.push_back(h_METParaSmeared);

   h_METxSmeared = new TH1F("METxSmeared", "MET x Smeared", nbins, metParaMin,metParaMax);
   h_METxSmeared->SetXTitle("M_{ET} x");
   h_METxSmeared->Sumw2();
   histogramsSmeared.push_back(h_METxSmeared);

   h_METySmeared = new TH1F("METySmeared", "MET y Smeared", nbins, metParaMin,metParaMax);
   h_METySmeared->SetXTitle("M_{ET} y");
   h_METySmeared->Sumw2();
   histogramsSmeared.push_back(h_METySmeared);

   
}

Bool_t METSmearing::Process(Long64_t entry) {
   // The Process() function is called for each entry in the tree (or possibly
   // keyed object in the case of PROOF) to be processed. The entry argument
   // specifies which entry in the currently loaded tree is to be processed.
   // It can be passed to either METSmearing::GetEntry() or TBranch::GetEntry()
   // to read either all or the required parts of the data. When processing
   // keyed objects with PROOF, the object is already loaded and is available
   // via the fObject pointer.
   //
   // This function should contain the "body" of the analysis. It can contain
   // simple or elaborate selection criteria, run algorithms on the data
   // of the event and typically fill histograms.
   //
   // The processing can be stopped by calling Abort().
   //
   // Use fStatus to set the return value of TTree::Process().
   //
   // The return value is currently not used.
   
   ++TotalEvents;
   
   GetEntry(entry);
   
   if (TotalEvents % 10000 == 0)
      cout << "Next event -----> " << TotalEvents << endl;
   
   BuildEvent();
   
   //double MuonPtCut = 25.;
   double MuonPtCut10 = 10.;
   double MuonRelIsoCut = 0.15;

   if(!triggerIsoMu24) return kFALSE;

   int N_IsoMuon10 = 0;
   int N_Muon10 = 0;

   int N_IsoMuon25 = 0;
   int N_Muon25 = 0;

   MyMuon *muoniso1, *muoniso2;
   MyMuon *muon1, *muon2;

   // global distributions
   
   for (vector<MyMuon>::iterator jt = Muons.begin(); jt != Muons.end(); ++jt) {
     
     if (jt->Pt()>MuonPtCut10) {
       ++N_Muon10;     

       if (N_Muon10 == 1) muon1 = &(*jt);
       if (N_Muon10 == 2) muon2 = &(*jt);

       if (jt->IsIsolated(MuonRelIsoCut)){
	 ++N_IsoMuon10;
       
	 if (N_IsoMuon10 == 1) muoniso1 = &(*jt);
	 if (N_IsoMuon10 == 2) muoniso2 = &(*jt);
	 
       }
       
     }
     
     if (jt->Pt()>MuonPtCut) {
       
       ++N_Muon25;
       
       
       if (jt->IsIsolated(MuonRelIsoCut)) {
	 ++N_IsoMuon25;
       }
     }
   }


   float smear1;
   float smear2;
   
   if(SmearVal<=0.){
     smear1=0;
     smear2=0;
   }else{	 
     smear1=rand.Gaus(0,SmearVal);
     smear2=rand.Gaus(0,SmearVal);
   }   
   bool doFill=false;
     
   if(doW){ 
     if (N_IsoMuon25 == 1 && N_IsoMuon10<2) { 
       doFill=true;
     }
   }else if(doZ){       
     if (N_IsoMuon25==2 && (*muoniso1 + *muoniso2).M()>70 &&  (*muoniso1 + *muoniso2).M()<110 ){
       doFill=true;
     }
   }else if (doQCD){

     if (N_IsoMuon10==0){   
       doFill=true;
     }
   }

   if(doFill){
     
     h_MET->Fill(met.Pt(), EventWeight);
     h_METx->Fill(met.Px(), EventWeight);
     h_METy->Fill(met.Py(), EventWeight);
     
     float metPerp, metPara;

     if(!doQCD){

     float phi=muon1->DeltaPhi(met);
     float metPlep=met.Px()*muon1->Px()+met.Py()*muon1->Py();
     float cosTheta=metPlep/(met.Pt()*muon1->Pt());
     metPara= met.Pt()*cosTheta;
     float metPlepRot=-met.Px()*muon1->Py()+met.Py()*muon1->Px();
     float cosTheta2=metPlepRot/(met.Pt()*muon1->Pt());
     metPerp= met.Pt()*cosTheta2;
     }else{
       
       // Rot pi/4
       //X'= -Y
       //Y'= X
       metPerp=(1/sqrt(2))*  (met.Px()-   met.Py());
       metPara=(1/sqrt(2))*  (met.Px()+   met.Py());

     }       
     
     h_METPara->Fill(metPara, EventWeight);
     h_METPerp->Fill(metPerp, EventWeight);
     
     float metParaSmear=metPara+smear1;
     float metPerpSmear=metPerp+smear2;
     float metXSmear=met.Px()+smear1;
     float metYSmear=met.Py()+smear2;
     
     float metSmear=sqrt(metParaSmear*metParaSmear+metPerpSmear*metPerpSmear);
     
     h_METParaSmeared->Fill(metParaSmear, EventWeight);
     h_METPerpSmeared->Fill(metPerpSmear, EventWeight);
     h_METxSmeared->Fill(metXSmear, EventWeight);
     h_METySmeared->Fill(metYSmear, EventWeight);
     h_METSmeared->Fill(metSmear, EventWeight);
     
   }   
   return kTRUE;
}

void METSmearing::SlaveTerminate() {
   // The SlaveTerminate() function is called after all entries or objects
   // have been processed. When running with PROOF SlaveTerminate() is called
   // on each slave server.
   
}

void METSmearing::Terminate() {
   // The Terminate() function is the last function to be called during
   // a query. It always runs on the client, it can be used to present
   // the results graphically or save the results to file.
   
}
