#define MyAnalysis_cxx
// The class definition in MyAnalysis.h has been generated automatically
// by the ROOT utility TTree::MakeSelector(). This class is derived
// from the ROOT class TSelector. For more information on the TSelector
// framework see $ROOTSYS/README/README.SELECTOR or the ROOT User Manual.

// The following methods are defined in this file:
//    Begin():        called every time a loop on the tree starts,
//                    a convenient place to create your histograms.
//    SlaveBegin():   called after Begin(), when on PROOF called only on the
//                    slave servers.
//    Process():      called for each event, in this function you decide what
//                    to read and fill your histograms.
//    SlaveTerminate: called at the end of the loop on the tree, when on PROOF
//                    called only on the slave servers.
//    Terminate():    called at the end of the loop on the tree,
//                    a convenient place to draw/fit your histograms.
//
// To use this file, try the following session on your Tree T:
//
// Root > T->Process("MyAnalysis.C")
// Root > T->Process("MyAnalysis.C","some options")
// Root > T->Process("MyAnalysis.C+")
//

#include "MyAnalysis.h"
#include <iostream>
#include <TH1F.h>
#include <TLatex.h>

using namespace std;

void MyAnalysis::BuildEvent() {
   
   Muons.clear();
   for (int i = 0; i < NMuon; ++i) {
      MyMuon muon(Muon_Px[i], Muon_Py[i], Muon_Pz[i], Muon_E[i]);
      muon.SetIsolation(Muon_Iso[i]);
      muon.SetCharge(Muon_Charge[i]);
      Muons.push_back(muon);
   }
   
   Electrons.clear();
   for (int i = 0; i < NElectron; ++i) {
      MyElectron electron(Electron_Px[i], Electron_Py[i], Electron_Pz[i], Electron_E[i]);
      electron.SetIsolation(Electron_Iso[i]);
      electron.SetCharge(Electron_Charge[i]);
      Electrons.push_back(electron);
   }
   
   Photons.clear();
   for (int i = 0; i < NPhoton; ++i) {
      MyPhoton photon(Photon_Px[i], Photon_Py[i], Photon_Pz[i], Photon_E[i]);
      photon.SetIsolation(Photon_Iso[i]);
      Photons.push_back(photon);
   }
   
   Jets.clear();
   for (int i = 0; i < NJet; ++i) {
      MyJet jet(Jet_Px[i], Jet_Py[i], Jet_Pz[i], Jet_E[i]);
      jet.SetBTagDiscriminator(Jet_btag[i]);
      jet.SetJetID(Jet_ID[i]);
      Jets.push_back(jet);
   }
   
   hadB.SetXYZM(MChadronicBottom_px, MChadronicBottom_py, MChadronicBottom_pz, 4.8);
   lepB.SetXYZM(MCleptonicBottom_px, MCleptonicBottom_py, MCleptonicBottom_pz, 4.8);
   hadWq.SetXYZM(MChadronicWDecayQuark_px, MChadronicWDecayQuark_py, MChadronicWDecayQuark_pz, 0.0);
   hadWqb.SetXYZM(MChadronicWDecayQuarkBar_px, MChadronicWDecayQuarkBar_py, MChadronicWDecayQuarkBar_pz, 0.0);
   lepWl.SetXYZM(MClepton_px, MClepton_py, MClepton_pz, 0.0);
   lepWn.SetXYZM(MCneutrino_px, MCneutrino_py, MCneutrino_pz, 0.0);
   met.SetXYZM(MET_px, MET_py, 0., 0.);
   

   EventWeight *= weight_factor;
   
}

void MyAnalysis::Begin(TTree * /*tree*/) {
   // The Begin() function is called at the start of the query.
   // When running with PROOF Begin() is only called on the client.
   // The tree argument is deprecated (on PROOF 0 is passed).
   TString option = GetOption();
   

}

void MyAnalysis::SlaveBegin(TTree * /*tree*/) {
   // The SlaveBegin() function is called after the Begin() function.
   // When running with PROOF SlaveBegin() is called on each slave server.
   // The tree argument is deprecated (on PROOF 0 is passed).
   
   TString option = GetOption();
  
   // global distributions   
   
   h_NMuon10Iso = new TH1F("NMuon10Iso", "Number of muons", 7, 0, 7);
   h_NMuon10Iso->SetXTitle("N Isolated Muons with pT>10 GeV");
   h_NMuon10Iso->Sumw2();
   histograms.push_back(h_NMuon10Iso);

   h_NMuon25Iso = new TH1F("NMuon25Iso", "Number of muons", 7, 0, 7);
   h_NMuon25Iso->SetXTitle("N Isolated Muons with pT>25 GeV");
   h_NMuon25Iso->Sumw2();
   histograms.push_back(h_NMuon25Iso);

   h_MET = new TH1F("METAll", "MET all events", 100, 0, 100);
   h_MET->SetXTitle("M_{ET} all events");
   h_MET->Sumw2();
   histograms.push_back(h_MET);

   // for events with 0 muon
   h_METNoMu = new TH1F("METNoMu", "MET no muon", 100, 0, 100);
   h_METNoMu->SetXTitle("M_{ET} (no muon)");
   h_METNoMu->Sumw2();
   histograms.push_back(h_METNoMu);
 
   h_METxNoMu = new TH1F("METxNoMu", "METx No Muon", 100, -50, 50);
   h_METxNoMu->SetXTitle("M_{ETx} (no muon)");
   h_METxNoMu->Sumw2();
   histograms.push_back(h_METxNoMu);

   h_METyNoMu = new TH1F("METyNoMu", "METy (single muon)", 100, -50, 50);
   h_METyNoMu->SetXTitle("M_{ETy} (no muon)");
   h_METyNoMu->Sumw2();
   histograms.push_back(h_METyNoMu);


   // for single muon events 1 muon > 25 GeV, no other isolated muon > 10 GeV

   h_METMuIso = new TH1F("METMuIso", "M_{ET} (single muon)", 100, 0, 100);
   h_METMuIso->SetXTitle("M_{ET} (single muon)");
   h_METMuIso->Sumw2();
   histograms.push_back(h_METMuIso);

   h_METPerpMuIso = new TH1F("METPerpMuIso", "MET single muon", 100, -50, 50);
   h_METPerpMuIso->SetXTitle("M_{ET} #perp (single muon)");
   h_METPerpMuIso->Sumw2();
   histograms.push_back(h_METPerpMuIso);

   h_METParaMuIso = new TH1F("METParaMuIso", "MET single muon", 100, -150, 50);
   h_METParaMuIso->SetXTitle("M_{ET} #parallel (single muon)");
   h_METParaMuIso->Sumw2();
   histograms.push_back(h_METParaMuIso);



   // for dimuon events: 1 muon > 25 GeV, 1 other isolated muon PT > 10 GeV

   h_METmumu = new TH1F("METmumu", "MET double muon", 100, 0, 100);
   h_METmumu->SetXTitle("M_{ET}  (double muon)");
   h_METmumu->Sumw2();
   histograms.push_back(h_METmumu);

   h_METPerpmumu = new TH1F("METPerpmumu", "MET para double muon", 100, -50, 50);
   h_METPerpmumu->SetXTitle("M_{ET} #perp  (double muon)");
   h_METPerpmumu->Sumw2();
   histograms.push_back(h_METPerpmumu);

   h_METParamumu = new TH1F("METParamumu", "MET para double muon", 100, -50, 50);
   h_METParamumu->SetXTitle("M_{ET} #parallel (double muon)");
   h_METParamumu->Sumw2();
   histograms.push_back(h_METParamumu);

}

Bool_t MyAnalysis::Process(Long64_t entry) {
   // The Process() function is called for each entry in the tree (or possibly
   // keyed object in the case of PROOF) to be processed. The entry argument
   // specifies which entry in the currently loaded tree is to be processed.
   // It can be passed to either MyAnalysis::GetEntry() or TBranch::GetEntry()
   // to read either all or the required parts of the data. When processing
   // keyed objects with PROOF, the object is already loaded and is available
   // via the fObject pointer.
   //
   // This function should contain the "body" of the analysis. It can contain
   // simple or elaborate selection criteria, run algorithms on the data
   // of the event and typically fill histograms.
   //
   // The processing can be stopped by calling Abort().
   //
   // Use fStatus to set the return value of TTree::Process().
   //
   // The return value is currently not used.
   
   ++TotalEvents;
   
   GetEntry(entry);
   
   if (TotalEvents % 10000 == 0)
      cout << "Next event -----> " << TotalEvents << endl;
   
   BuildEvent();
   
   double MuonPtCut = 25.;
   double MuonPtCut10 = 10.;
   double MuonRelIsoCut = 0.15;
   
   
   //////////////////////////////

   if(!triggerIsoMu24) return kFALSE;

   int N_IsoMuon10 = 0;
   int N_Muon10 = 0;

   int N_IsoMuon25 = 0;
   int N_Muon25 = 0;

   MyMuon *muoniso1, *muoniso2;
   MyMuon *muon1, *muon2;

   // global distributions

   for (vector<MyMuon>::iterator jt = Muons.begin(); jt != Muons.end(); ++jt) {
     
     if (jt->Pt()>MuonPtCut10) {
       ++N_Muon10;     
       std::pair<MyMuon*, float> myPair=std::make_pair(&(*jt),(&(*jt))->Pt());
       //muons10.push_back(myPair);

       if (N_Muon10 == 1) muon1 = &(*jt);
       if (N_Muon10 == 2) muon2 = &(*jt);

       if (jt->IsIsolated(MuonRelIsoCut)){
	 ++N_IsoMuon10;
       
	 if (N_IsoMuon10 == 1) muoniso1 = &(*jt);
	 if (N_IsoMuon10 == 2) muoniso2 = &(*jt);

       }
     
     }
     
     if (jt->Pt()>MuonPtCut) {
       
       ++N_Muon25;       
       
       if (jt->IsIsolated(MuonRelIsoCut)) {
	 ++N_IsoMuon25;
       }
     }
   }   

   h_NMuon10Iso->Fill(N_IsoMuon10,EventWeight);
   h_NMuon25Iso->Fill(N_IsoMuon25,EventWeight);
   h_MET->Fill(met.Pt(),EventWeight);
   
   
   // for events with > 1 muon of PT 25 GeV
   // isolation

   if (N_Muon10==0){
     h_METNoMu->Fill(met.Pt(), EventWeight);
     h_METxNoMu->Fill(met.Px(), EventWeight);
     h_METyNoMu->Fill(met.Py(), EventWeight);
   }
   
   if (N_IsoMuon25 == 1 ) {
     
     h_METMuIso->Fill(met.Pt(), EventWeight);
     
     float metPlep=met.Px()*muon1->Px()+met.Py()*muon1->Py();
     float cosTheta=metPlep/(met.Pt()*muon1->Pt());
     float metPara= met.Pt()*cosTheta;
     
     // Rot pi/2
     //X'= -Y
     //Y'= X
     
     float metPlepRot=-met.Px()*muon1->Py()+met.Py()*muon1->Px();
     float cosTheta2=metPlepRot/(met.Pt()*muon1->Pt());
     float metPerp= met.Pt()*cosTheta2;
     
     h_METParaMuIso->Fill(metPara, EventWeight);
     h_METPerpMuIso->Fill(metPerp, EventWeight);

   }

    
   // for dimuon events: 1 muon > 25 GeV, 1 other isolated muon PT > 10 GeV

   if (N_IsoMuon25>=1 && N_IsoMuon10>=2) {
     

     if((*muoniso1 + *muoniso2).M()>70 &&  (*muoniso1 + *muoniso2).M()<110 ){
     
       h_METmumu->Fill(met.Pt(), EventWeight);
       
       float metPlep=met.Px()*muoniso1->Px()+met.Py()*muoniso1->Py();
       float cosTheta=metPlep/(met.Pt()*muoniso1->Pt());
       float metPara= met.Pt()*cosTheta;       
       float metPlepRot=-met.Px()*muoniso1->Py()+met.Py()*muoniso1->Px();
       float cosTheta2=metPlepRot/(met.Pt()*muoniso1->Pt());
       float metPerp= met.Pt()*cosTheta2;
       
       h_METParamumu->Fill(metPara, EventWeight);
       h_METPerpmumu->Fill(metPerp, EventWeight);
       
     }
   }

      
   //////////////////////////////
   
   return kTRUE;
}

void MyAnalysis::SlaveTerminate() {
   // The SlaveTerminate() function is called after all entries or objects
   // have been processed. When running with PROOF SlaveTerminate() is called
   // on each slave server.
   
}

void MyAnalysis::Terminate() {
   // The Terminate() function is the last function to be called during
   // a query. It always runs on the client, it can be used to present
   // the results graphically or save the results to file.
   
}
