/*
 * Plotter.h
 *
 *  Created on: 25.06.2012
 *      Author: csander
 */

#ifndef PLOTTER_H_
#define PLOTTER_H_

#include <vector>
#include <string>
#include <iostream>

#include <TH1F.h>
#include <TStyle.h>
#include <THStack.h>
#include <TCanvas.h>
#include <TLegend.h>
#include <TROOT.h>
#include <iostream>

using namespace std;

class Plotter {
public:
	Plotter();
	virtual ~Plotter();
	void SetData(std::vector<TH1F*> v, std::string n){
		data.push_back(v);
		data_names.push_back(n);
		N_histos = v.size();
	}
	void ClearData(){
		data.clear();
		data_names.clear();
	}
	void AddBg(std::vector<TH1F*> v, std::string n){
		bg.push_back(v);
		bg_names.push_back(n);
		N_histos = v.size();
	}
	void AddBgNormalized(std::vector<TH1F*> v, std::string n){
	  std::vector<TH1F*> bgnormalized;
	  bgnorm_names.push_back(n);
	  if(v.size()!=data.at(0).size()) abort();
	  for(int j=0;j<v.size();j++){
	    TH1F* hist = (TH1F*) v[j]->DrawCopy();
	    float norm = data.at(0).at(j)->Integral()/v[j]->Integral();
	    cout<<" norm=" <<norm<< endl;
	    for(int i=0;i<hist->GetNbinsX();i++){
	      float content=hist->GetBinContent(j);
	      float error=hist->GetBinError(j);
	      hist->SetBinContent(i+1,content*norm);
	      hist->SetBinError(i+1,error*norm);
	    }
	    TString name =  v[j]->GetName();
	    name+="Norm";
	    hist->SetName(name);
	    bgnormalized.push_back(hist);	      
	  }
	  bgnorm.push_back( bgnormalized );
	}
	void ClearBg(){
		bg.clear();
		bg_names.clear();
	}
	void ClearBgNorm(){
		bgnorm.clear();
		bgnorm_names.clear();
	}
	void AddSig(std::vector<TH1F*> v, std::string n){
		signal.push_back(v);
		signal_names.push_back(n);
		N_histos = v.size();
	}
	void ClearSig(){
		signal.clear();
		signal_names.clear();
	}
	void Plot(std::string filename = "result.pdf", bool DrawLog = true);
	void PlotNorm(std::string filename = "resultNorm.pdf", bool DrawLog = true);

private:
	std::vector < std::vector<TH1F*> > data;
	std::vector < std::vector<TH1F*> > bg;
	std::vector < std::vector<TH1F*> > signal;
	std::vector < std::vector<TH1F*> > bgnorm;

	std::vector < std::string > data_names;
	std::vector < std::string > bg_names;
	std::vector < std::string > bgnorm_names;
	std::vector < std::string > signal_names;

	int N_histos;

};

#endif /* PLOTTER_H_ */
