#include "MyAnalysis.h"
#include "CutAndCount.h"
#include "METSmearing.h"
#include "MakeTree.h"
#include "Plotter.h"
#include <iostream>
#include <TChain.h>
#include <TLegend.h>
#include <TH1.h>
#include <TGraphAsymmErrors.h>
#include <string>




/////////////////////////////////////////
// Functions to get the CMS paper plots 
/////////////////////////////////////////
// getPaperPlot: returns TH1 with CMS experimental results
// getPaperPlotTheo: returns TGraphAsymErrors with theory expeactations
// DrawPaperPlot: draws both plots

// Function to get the CMS paper plots (experimental results)
//=============================================================

TH1F* getPaperPlot(float ptCut){

  float binsPlot[9];
  binsPlot[0]=-0.1;
  binsPlot[1]=0.0;
  binsPlot[2]=0.4;
  binsPlot[3]=0.8;
  binsPlot[4]=1.2;
  binsPlot[5]=1.5;
  binsPlot[6]=1.8;
  binsPlot[7]=2.1;
  binsPlot[8]=2.7;

  float valuesFromPaper[6];
  float errorsFromPaper[6];

  if(ptCut==25){
    valuesFromPaper[0]=0.147;
    valuesFromPaper[1]=0.159;
    valuesFromPaper[2]=0.184;
    valuesFromPaper[3]=0.207;
    valuesFromPaper[4]=0.231;
    valuesFromPaper[5]=0.253;
    
    errorsFromPaper[0]=0.010;
    errorsFromPaper[1]=0.0092;
    errorsFromPaper[2]=0.0125;
    errorsFromPaper[3]=0.0122;
    errorsFromPaper[4]=0.0136;
    errorsFromPaper[5]=0.0161;
    
  }else{
    
    valuesFromPaper[0]=0.131;
    valuesFromPaper[1]=0.139;
    valuesFromPaper[2]=0.158;
    valuesFromPaper[3]=0.185;
    valuesFromPaper[4]=0.202;
    valuesFromPaper[5]=0.231;
    
    errorsFromPaper[0]=0.0122;
    errorsFromPaper[1]=0.0114;
    errorsFromPaper[2]=0.0147;
    errorsFromPaper[3]=0.0136;
    errorsFromPaper[4]=0.0144;
    errorsFromPaper[5]=0.0174;
  }
  
  TString nameh="AsymPap_";
  nameh+=ptCut;


  TH1F *histAsym = new TH1F(nameh.Data(),nameh.Data(),8,binsPlot);
  
  for (int i=0;i<6;i++){
    histAsym->SetBinContent(i+2,valuesFromPaper[i]);
    histAsym->SetBinError(i+2,errorsFromPaper[i]);
  }
  
  histAsym->SetTitle("");
  histAsym->SetXTitle("Lepton #eta");
  histAsym->SetYTitle("Lepton charge asymetry");  
  histAsym->SetMarkerStyle(21);
  histAsym->GetYaxis()->SetRangeUser(0.02,0.34);
  histAsym->GetYaxis()->CenterTitle();
  histAsym->GetXaxis()->CenterTitle();
  return histAsym;
  
}

// Function to get the CMS paper plots (theory expectations)
//=============================================================

vector<TGraphAsymmErrors*> getPaperPlotTheo(float ptCut){

  float binsPlot[7];
  binsPlot[0]=0.0;
  binsPlot[1]=0.4;
  binsPlot[2]=0.8;
  binsPlot[3]=1.2;
  binsPlot[4]=1.5;
  binsPlot[5]=1.8;
  binsPlot[6]=2.1;

  float x[6];
  float ex[6];
  for (int i=0;i<6;i++){
    x[i]=binsPlot[i]+0.5*(binsPlot[i+1]-binsPlot[i]);
    ex[i]=0;
  }


  float valuesFromPaper[2][6]; // 0 = RESBOS, 1 = MCFM
  float errorsLowFromPaper[2][6]; // 0 = RESBOS, 1 = MCFM
  float errorsHighFromPaper[2][6]; // 0 = RESBOS, 1 = MCFM

  if(ptCut==25){

    valuesFromPaper[0][0]=0.157;
    valuesFromPaper[0][1]=0.169;
    valuesFromPaper[0][2]=0.193;
    valuesFromPaper[0][3]=0.22;
    valuesFromPaper[0][4]=0.246;
    valuesFromPaper[0][5]=0.265;
    
    errorsHighFromPaper[0][0]=0.8;
    errorsHighFromPaper[0][1]=0.9;
    errorsHighFromPaper[0][2]=0.8;
    errorsHighFromPaper[0][3]=0.8;
    errorsHighFromPaper[0][4]=0.8;
    errorsHighFromPaper[0][5]=0.8;

    errorsLowFromPaper[0][0]=1.0;
    errorsLowFromPaper[0][1]=1.0;
    errorsLowFromPaper[0][2]=1.1;
    errorsLowFromPaper[0][3]=1.1;
    errorsLowFromPaper[0][4]=1.1;
    errorsLowFromPaper[0][5]=1.0;

    valuesFromPaper[1][0]=0.153;
    valuesFromPaper[1][1]=0.167;
    valuesFromPaper[1][2]=0.192;
    valuesFromPaper[1][3]=0.220;
    valuesFromPaper[1][4]=0.245;
    valuesFromPaper[1][5]=0.263;
    
    errorsHighFromPaper[1][0]=0.8;
    errorsHighFromPaper[1][1]=0.9;
    errorsHighFromPaper[1][2]=0.8;
    errorsHighFromPaper[1][3]=0.8;
    errorsHighFromPaper[1][4]=0.8;
    errorsHighFromPaper[1][5]=0.8;

    errorsLowFromPaper[1][0]=1.0;
    errorsLowFromPaper[1][1]=1.0;
    errorsLowFromPaper[1][2]=1.1;
    errorsLowFromPaper[1][3]=1.1;
    errorsLowFromPaper[1][4]=1.1;
    errorsLowFromPaper[1][5]=1.0;
    
  }else{
    
    valuesFromPaper[0][0]=0.134;
    valuesFromPaper[0][1]=0.146;
    valuesFromPaper[0][2]=0.169;
    valuesFromPaper[0][3]=0.196;
    valuesFromPaper[0][4]=0.222;
    valuesFromPaper[0][5]=0.245;
    
    errorsHighFromPaper[0][0]=0.7;
    errorsHighFromPaper[0][1]=0.8;
    errorsHighFromPaper[0][2]=0.8;
    errorsHighFromPaper[0][3]=0.8;
    errorsHighFromPaper[0][4]=0.8;
    errorsHighFromPaper[0][5]=0.8;

    errorsLowFromPaper[0][0]=0.9;
    errorsLowFromPaper[0][1]=0.8;
    errorsLowFromPaper[0][2]=1.0;
    errorsLowFromPaper[0][3]=1.0;
    errorsLowFromPaper[0][4]=1.1;
    errorsLowFromPaper[0][5]=1.1;

    valuesFromPaper[1][0]=0.131;
    valuesFromPaper[1][1]=0.145;
    valuesFromPaper[1][2]=0.168;
    valuesFromPaper[1][3]=0.194;
    valuesFromPaper[1][4]=0.219;
    valuesFromPaper[1][5]=0.241;
    
    errorsHighFromPaper[1][0]=0.7;
    errorsHighFromPaper[1][1]=0.8;
    errorsHighFromPaper[1][2]=0.8;
    errorsHighFromPaper[1][3]=0.8;
    errorsHighFromPaper[1][4]=0.8;
    errorsHighFromPaper[1][5]=0.8;

    errorsLowFromPaper[1][0]=0.9;
    errorsLowFromPaper[1][1]=0.8;
    errorsLowFromPaper[1][2]=1.0;
    errorsLowFromPaper[1][3]=1.0;
    errorsLowFromPaper[1][4]=1.1;
    errorsLowFromPaper[1][5]=1.1;

  }
  float yR[6];
  float yM[6];
  float eyRL[6];
  float eyML[6];
  float eyRH[6];
  float eyMH[6];
  
  for (int i=0;i<6;i++){
    yR[i]= valuesFromPaper[0][i];
    yM[i]= valuesFromPaper[1][i];
    eyRL[i]=errorsLowFromPaper[0][i]/100.;
    eyRH[i]=errorsHighFromPaper[0][i]/100.;
    eyML[i]=errorsLowFromPaper[1][i]/100.;
    eyMH[i]=errorsHighFromPaper[1][i]/100.;
  }

  TGraphAsymmErrors *histR = new TGraphAsymmErrors(6,x,yR,ex,ex,eyRL,eyRH);
  TGraphAsymmErrors *histM = new TGraphAsymmErrors(6,x,yM,ex,ex,eyML,eyMH);
  histR->SetFillColor(5);
  histM->SetFillColor(8);
  histR->SetLineColor(kBlue);
  histM->SetLineColor(kBlue);
  histR->SetLineStyle(kDashed);
  histM->SetLineStyle(kDashed);
  histR->SetLineWidth(2);
  histM->SetLineWidth(2);


  vector<TGraphAsymmErrors*> res;
  res.push_back(histR);
  res.push_back(histM);
  return res;

}

// Function to draw the CMS paper plots (exp+theory)
//==================================================

void SetStyle(){

  gStyle->SetOptStat(0);
  gStyle->SetOptTitle(0);

  gStyle->SetTitleOffset(1.0,"x");
  gStyle->SetTitleOffset(1.2,"y");

  gStyle->SetPadBottomMargin(0.1);
  gStyle->SetPadTopMargin(0.05);
  gStyle->SetPadLeftMargin(0.10);
  gStyle->SetPadRightMargin(0.05);
  
}

void DrawPaperPlot(){

  SetStyle();
  
  TH1F *Paper_30  =  getPaperPlot(30);
  TH1F *Paper_25  =  getPaperPlot(25);

  vector<TGraphAsymmErrors*> theo25=getPaperPlotTheo(25);
  vector<TGraphAsymmErrors*> theo30=getPaperPlotTheo(30);

  Paper_25->SetMarkerColor(kBlack);
  Paper_25->SetMarkerColor(kBlack);
  Paper_30->SetMarkerColor(kBlack);
  Paper_30->SetMarkerColor(kBlack);
  Paper_25->SetLineColor(kBlack);
  Paper_30->SetLineColor(kBlack);

  TLegend *leg25= new TLegend(0.25,0.7,0.5,0.85); 
  leg25->SetBorderSize(0);
  leg25->SetFillStyle(0);
  leg25->SetTextFont(42);
  leg25->SetHeader("pT > 25 GeV");
  leg25->AddEntry(Paper_25,"CMS paper","lm"); 
  leg25->AddEntry(theo25[0],"Resbos","fl"); 
  leg25->AddEntry(theo25[1],"MCFM","fl"); 
  
  TLegend *leg30= new TLegend(0.25,0.7,0.5,0.85); 
  leg30->SetBorderSize(0);
  leg30->SetFillStyle(0);
  leg30->SetTextFont(42);
  leg30->SetHeader("pT > 30 GeV");
  leg30->AddEntry(Paper_30,"CMS paper","lm"); 
  leg30->AddEntry(theo30[0],"Resbos","fl"); 
  leg30->AddEntry(theo30[1],"MCFM","fl"); 
  
  TCanvas *can25 = new TCanvas("can25","can25");
  Paper_25->Draw("E1");
  theo25[0]->Draw("3same");
  theo25[0]->Draw("Lsame");
  theo25[1]->Draw("3same");
  theo25[1]->Draw("Lsame");
  Paper_25->Draw("E1Same");
  leg25->Draw("same");
  can25->Print("plots/Paper25.pdf","pdf");
  

  TCanvas *can30 = new TCanvas("can30","can30");
  Paper_30->Draw("E1");
  theo30[0]->Draw("3same");
  theo30[0]->Draw("Lsame");
  theo30[1]->Draw("3same");
  theo30[1]->Draw("Lsame");
  Paper_30->Draw("E1same");
  leg30->Draw("same");
  can30->Print("plots/Paper30.pdf","pdf");

}

////////////////////////////////
// Exercice 1.1: Do basic plots
////////////////////////////////

void BasicStackPlots(){


  float lumi = 50.;
   
   MyAnalysis *A = new MyAnalysis();
   TChain* ch = new TChain("events");
   ch->Add("files/data.root"); 
   ch->Process(A);
   
   MyAnalysis *B = new MyAnalysis();
   TChain* ch2 = new TChain("events");
   ch2->Add("files/ttbar.root");
   ch2->Process(B);
   
   MyAnalysis *C = new MyAnalysis();
   TChain* ch3 = new TChain("events");
   ch3->Add("files/wjets.root");
   ch3->Process(C);
   
   MyAnalysis *D = new MyAnalysis();
   TChain* ch4 = new TChain("events");
   ch4->Add("files/dy.root");
   ch4->Process(D);
   
   MyAnalysis *E = new MyAnalysis();
   TChain* ch5 = new TChain("events");
   ch5->Add("files/ww.root");
   ch5->Process(E);

   MyAnalysis *F = new MyAnalysis();
   TChain* ch6 = new TChain("events");
   ch6->Add("files/wz.root");
   ch6->Process(F);

   MyAnalysis *G = new MyAnalysis();
   TChain* ch7 = new TChain("events");
   ch7->Add("files/zz.root");
   ch7->Process(G);

   MyAnalysis *H = new MyAnalysis();
   TChain* ch8 = new TChain("events");
   ch8->Add("files/qcd.root");
   ch8->Process(H);
   
   MyAnalysis *I = new MyAnalysis();
   TChain* ch9 = new TChain("events");
   ch9->Add("files/single_top.root");
   ch9->Process(I);
   
   Plotter P;
   P.SetData(A->histograms, std::string("Data"));
   P.AddBg(B->histograms, std::string("TTbar"));
   P.AddBg(C->histograms, std::string("Wjets"));
   P.AddBg(D->histograms, std::string("DY"));
   P.AddBg(H->histograms, std::string("QCD"));
   P.AddBg(E->histograms, std::string("WW"));
   P.AddBg(F->histograms, std::string("WZ"));
   P.AddBg(G->histograms, std::string("ZZ"));
   P.AddBg(I->histograms, std::string("Single Top"));
  
   P.Plot(string("plots/BasicStackPlots.pdf"),false);
   P.Plot(string("plots/BasicStackPlotsLog.pdf"),true);

 
}

///////////////////////////////////////////////////
// Exercice 1.2: Compute raw cut and count asymetry
///////////////////////////////////////////////////

// Function getAsym: computes the raw cut and count asymetry for a given MetCut and PtCut using class CutAndCount
// ================================================================================================================

TH1F* getAsym(float MetCut, float PtCut, bool doW){

  // Compute asymetry for pt cut 25
  CutAndCount *A = new CutAndCount();
  TChain* ch = new TChain("events");
  ch->Add("files/data.root"); 
  A->SetPtCut(PtCut);
  A->SetMetCut(MetCut);
  if(!doW)  A->SetMetCut(0);
  A->SetDoW(doW);
  ch->Process(A);

  vector<float> etaLow=A->EtaLow;
  vector<float> etaHigh=A->EtaHigh;
  
  vector<float> asym=A->Asymetry;
  vector<float> error=A->AsymetryError;
  
 
  int nbins=6;
  float bins[7];
  float binsPlot[9];
  binsPlot[0]=-0.1;
  binsPlot[8]=2.7;

  for (int i=0;i<etaLow.size();i++){
    cout<<" Asym["<<etaLow[i]<<"<eta<"<<etaHigh[i]<<"] = "<<asym[i]<< " +/- "<< error[i]<< endl;
    bins[i]=etaLow[i];
    binsPlot[i+1]=bins[i];
  }
  bins[nbins]=etaHigh[nbins-1];
  binsPlot[nbins+1]=bins[nbins];
 
  TString nameh="Asym_";
  nameh+=PtCut;
  nameh+="_";
  nameh+=MetCut;

  TH1F *histAsym = new TH1F(nameh.Data(),nameh.Data(),8,binsPlot);
  
  for (int i=0;i<etaLow.size();i++){
    histAsym->SetBinContent(i+2,asym[i]);
    histAsym->SetBinError(i+2,error[i]);
  }

  histAsym->SetTitle("");
  histAsym->SetXTitle("Lepton #eta");
  histAsym->SetYTitle("Lepton charge asymetry");  
  histAsym->GetYaxis()->CenterTitle();
  histAsym->GetXaxis()->CenterTitle();
  histAsym->SetMarkerStyle(20);
  if(doW) histAsym->GetYaxis()->SetRangeUser(0.02,0.34); 
  else histAsym->GetYaxis()->SetRangeUser(-0.1,0.1); 

  return histAsym;
}

// Function getAsymMC: computes the raw cut and count asymetry for a given MetCut and PtCut on MC
// ===============================================================================================

  
//////////////////////////////////////////////////////////////////////////////////////////
// TODO: write this part taking advantage of class CutAndCount and using example getAsym
///////////////////////////////////////////////////////////////////////////////////////////

TH1F* getAsymMC(float MetCut, float PtCut, bool doW){

  cout<<" This has to be coded ! "<< endl;

  TH1F *histAsym;  
  return histAsym;
}

// Function DrawCutAndCountAsym: Draws cut and count asymetries and compares to CMS paper results
// ===============================================================================================
void DrawCutAndCountAsym(bool doW){

  SetStyle();

  TH1F *Asym_25_25 =  getAsym(25,25, doW);
  TH1F *Asym_30_25 =  getAsym(30,25, doW);
  TH1F *Asym_25_30 =  getAsym(25,30, doW);
  TH1F *Asym_30_30 =  getAsym(30,30, doW);

  TH1F *Paper_30  =  getPaperPlot(30);
  TH1F *Paper_25  =  getPaperPlot(25);

  Asym_25_25->SetMarkerColor(kRed);
  Asym_25_25->SetLineColor(kRed);
  Asym_25_30->SetMarkerColor(kBlue);
  Asym_25_30->SetLineColor(kBlue);
  Paper_25->SetMarkerColor(kBlack);
  Paper_25->SetMarkerColor(kBlack);

  Asym_30_25->SetMarkerColor(kRed);
  Asym_30_25->SetLineColor(kRed);
  Asym_30_30->SetMarkerColor(kBlue);
  Asym_30_30->SetLineColor(kBlue);
  Paper_30->SetMarkerColor(kBlack);
  Paper_30->SetMarkerColor(kBlack);

  TLegend *leg25= new TLegend(0.2,0.7,0.5,0.85); 
  leg25->SetBorderSize(0);
  leg25->SetFillStyle(0);
  leg25->SetTextFont(42);
  leg25->SetHeader("pT > 25 GeV");
  if(doW){ 
    leg25->AddEntry(Asym_25_25,"M_{ET} > 25 GeV","lm"); 
    leg25->AddEntry(Asym_25_30,"M_{ET} > 30 GeV","lm"); 
    leg25->AddEntry(Paper_25,"CMS paper","lm"); 
  }
  TLegend *leg30= new TLegend(0.2,0.7,0.5,0.85); 
  leg30->SetBorderSize(0);
  leg30->SetFillStyle(0);
  leg30->SetTextFont(42);
  leg30->SetHeader("pT > 30 GeV");
  if(doW){ 
    leg30->AddEntry(Asym_30_25,"M_{ET} > 25 GeV","lm"); 
    leg30->AddEntry(Asym_30_30,"M_{ET} > 30 GeV","lm"); 
    leg30->AddEntry(Paper_30,"CMS paper","lm"); 
  }
  TCanvas *can25 = new TCanvas("can25","can25");
  Asym_25_25->Draw("E1");
  if(doW) Asym_25_30->Draw("E1same");
  if(doW) Paper_25->Draw("E1same");
  if(doW) leg25->Draw("same");
  if(doW)
    can25->Print("plots/WAsym_CutAndCount_Pt25.pdf","pdf");
  else
    can25->Print("plots/ZAsym_CutAndCount.pdf","pdf");

  TCanvas *can30 = new TCanvas("can30","can30");
  Asym_30_25->Draw("E1");
  if(doW) Asym_30_30->Draw("E1same");
  if(doW) Paper_30->Draw("E1same");
  if(doW)leg30->Draw("same");
  if(doW)
    can30->Print("plots/WAsym_CutAndCount_Pt30.pdf","pdf");
  else  
    can30->Print("plots/ZAsym_CutAndCount.pdf","pdf");
 
}



// Function DrawCutAndCountAsymMC: Draws cut and count asymetries on MC and compares to MC result no met cut
// ===========================================================================================================
void DrawCutAndCountAsymMC(){
  
  ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  // TODO: write this part taking advantage of example DrawCutAndCountAsym to draw W MC asymetries for different MET cuts and pt cuts
  ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  SetStyle();

  cout<<" This has to be coded ! "<< endl;

  bool doW=true;
  
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Exercice 2.1: Determine MET smearing to be applied to MC on a Z sample and determine PDFs for 3 species
///////////////////////////////////////////////////////////////////////////////////////////////////////////

// Function fitMetSmearing: (2.1.1) Determines the best smearing for a given ptCut
// ===============================================================================

void fitMetSmearing(float ptCut=25){

  SetStyle();

  ////////////////////////////////////////////////////////////////////////////////////////////////////////
  // TODO: write this part taking advantage of class METSmearing which provides MET smeared distributions
  ////////////////////////////////////////////////////////////////////////////////////////////////////////

  // Example: Met Histos for DY data:
  METSmearing *A = new METSmearing();
  TChain* ch = new TChain("events");
  ch->Add("files/data.root"); 
  A->SetSmearing(0);
  A->SetPtCut(ptCut);
  A->SetNMuons(2);
  ch->Process(A);
  TH1F *METData2Mu=(TH1F*) A->histograms.at(0)->Clone("METData2Mu"); // This gives unsmeared met histogram(reference histogram)
  METData2Mu->SetLineWidth(3);
  METData2Mu->SetXTitle("M_{ET}");

  // Met Histos for DY MC:
  TChain* ch2 = new TChain("events");
  ch2->Add("files/dy.root"); 
  TH1F *METMC2Mu;
  TH1F *METMCSmeared2Mu;
  
  float smear=3;
  METSmearing *B = new METSmearing();
  B->SetNMuons(2);
  B->SetPtCut(ptCut);
  B->SetSmearing(smear);
  ch2->Process(B);
  METMCSmeared2Mu=(TH1F*) B->histogramsSmeared.at(0)->Clone("METMCSmeared2Mu"); // This gives smeared met histogram
  METMCSmeared2Mu->Scale(METData2Mu->Integral()/METMCSmeared2Mu->Integral());
  METMCSmeared2Mu->SetLineColor(2);    
  METMCSmeared2Mu->SetXTitle("M_{ET}");
  
  METMC2Mu=(TH1F*) B->histograms.at(0)->Clone("METMC2Mu"); // This gives unsmeared met histogram 
  METMC2Mu->Scale(METData2Mu->Integral()/METMC2Mu->Integral());
  METMC2Mu->SetLineColor(2);    
  METMC2Mu->SetXTitle("M_{ET}");
  
  float chi2=METMCSmeared2Mu->Chi2Test(METData2Mu,"NORMUUCHI2/NDF"); // This gives the chi2 to be minimized


}

// Function applyMetSmearing: (2.1.2) and (2.1.3) Applies best smearing and saves PDFs
// ===================================================================================

void applyMetSmearing(float ptCut){

  SetStyle();

  ////////////////////////////////////////////////////////////////////////////////////////////////////////
  // TODO: write this part taking advantage of class METSmearing which provides MET smeared distributions
  ////////////////////////////////////////////////////////////////////////////////////////////////////////

  TFile *PDF;
  if(ptCut==25) PDF = new TFile("results/PDF.root","RECREATE"); // the file where to save your PDFs
  else PDF = new TFile("results/PDF30.root","RECREATE"); // the file where to save your PDFs
  
  // your best smearing:
  float smearingFromZ=3; // Put here your best smearing ! ! !


  // EXAMPLE FOR W PDF:
  //===================

  // Met Histos for W MC and smeared MET for W MC:
  METSmearing *B = new METSmearing();
  TChain* ch2 = new TChain("events");
  ch2->Add("files/wjets.root"); 
  B->SetSmearing(smearingFromZ);
  B->SetPtCut(ptCut);
  B->SetNMuons(1);
  ch2->Process(B);
  TH1F *METW1Mu=(TH1F*) B->histograms.at(0)->Clone("METW1Mu");
  METW1Mu->SetXTitle("M_{ET}");
  TH1F *METWSmeared1Mu=(TH1F*) B->histogramsSmeared.at(0)->Clone("PDF_W");
  METWSmeared1Mu->SetXTitle("M_{ET}");

  METWSmeared1Mu->Write();
  PDF->Close();

}


// Function DrawPDF: (2.1.2) and (2.2.3) Draws the PDFs
// ====================================================

void DrawPDF(float ptCut){

  SetStyle();
  TString ls="pT >"; ls+=int(ptCut);ls+=" GeV";

  TFile *file;
  if(ptCut==25) file= new TFile("results/PDF.root");
  else file= new TFile("results/PDF30.root");
  TH1F *PDF_Z=(TH1F*)file->Get("PDF_Z");
  TH1F *PDF_W=(TH1F*)file->Get("PDF_W");
  TH1F *PDF_QCD=(TH1F*)file->Get("PDF_QCD");
  PDF_Z->Scale(1./PDF_Z->Integral());
  PDF_W->Scale(1./PDF_W->Integral());
  PDF_QCD->Scale(1./PDF_QCD->Integral());
  PDF_W->SetLineColor(kRed);
  PDF_W->SetLineWidth(2);
  PDF_Z->SetLineColor(kBlue);
  PDF_Z->SetLineWidth(2);
  PDF_QCD->SetLineColor(kBlack);
  PDF_QCD->SetLineWidth(2);


  TLegend *leg= new TLegend(0.5,0.7,0.8,0.85); 
  leg->SetBorderSize(0);
  leg->SetFillStyle(0);
  leg->SetTextFont(42);
  leg->SetHeader(ls.Data());
  leg->AddEntry(PDF_W,"PDF W","lm");
  leg->AddEntry(PDF_QCD,"PDF QCD","lm"); 
  leg->AddEntry(PDF_Z ,"PDF Z","lm"); 

  TCanvas *can = new TCanvas("can","can");
  PDF_QCD->Draw();
  PDF_W->Draw("same");
  PDF_Z->Draw("same");leg->Draw("same");
  if(ptCut==25) can->Print("plots/PDFs.pdf","pdf");
  else can->Print("plots/PDFs30.pdf","pdf");
}



////////////////////////////////////////
// Exercice 2.2: Maximum Likelihood Fit
////////////////////////////////////////

// Function getZfrac: (2.2.1) Determines the ratio between the total Z and W yields versus eta 
// ============================================================================================


void getZfrac(float PtCut){

  SetStyle();

  ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  // TODO: write this part taking advantage of class CutAndCount. Modify this class CutAndCount to deal also with Z changing bool doW
  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}

// Function makeTree: to make a simpler data tree for the fit (tree already done)
// ==============================================================================

void makeTree(){

  MakeTree *A = new MakeTree();
  TChain* ch = new TChain("events");
  ch->Add("files/data.root");
  ch->Process(A);
}

// NB: the rest of exercice 2 will be done in a separate macro.


int main() {

  // To draw CMS paper plot:
  //=========================
  //DrawPaperPlot();

  // Exercice 1.1:
  //===============
  //BasicStackPlots();

  // Exercice 1.2:
  //===============
  //DrawCutAndCountAsym(true);
  //DrawCutAndCountAsymMC();

  // Exercice 2.1:
  //===============

  //fitMetSmearing(25);  
  //fitMetSmearing(30); 

  //applyMetSmearing(25);
  //applyMetSmearing(30);

  //DrawPDF(25);
  //DrawPDF(30);
  
  // Exercice 2.2:
  //===============
  //getZfrac(25);
  //getZfrac(30);  

}
